@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
@@ -6,15 +6,15 @@
 #include "LocalListView.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "filter.h"
+#include "filter_manager.h"
 #include "file_utils.h"
+#include "infotext.h"
 #include "inputdialog.h"
 #include <algorithm>
 #include "dndobjects.h"
 #include "Options.h"
 #ifdef __WXMSW__
 #include "lm.h"
-#include <wx/msw/registry.h>
 #include "volume_enumerator.h"
 #endif
 #include "dragdropmanager.h"
@@ -27,8 +27,13 @@
 #include "timeformatting.h"
 
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/process.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
+#include <wx/menu.h>
+
+using namespace std::literals;
+
 class CLocalListViewDropTarget final : public CFileDropTarget<wxListCtrlEx>
 {
 public:
@@ -54,7 +59,7 @@ public:
 
 	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		def = CScrollableDropTarget<wxListCtrlEx>::FixupDragResult(def);
+		def = FixupDragResult(def);
 
 		if (def == wxDragError ||
 			def == wxDragNone ||
@@ -77,7 +82,7 @@ public:
 		}
 
 		std::wstring subdir;
-		int flags;
+		int flags{};
 		int hit = m_pLocalListView->HitTest(wxPoint(x, y), flags, 0);
 		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 			const CLocalFileData* const data = m_pLocalListView->GetData(hit);
@@ -101,25 +106,26 @@ public:
 			return wxDragError;
 		}
 
-		auto const format = m_pDataObject->GetReceivedFormat();
+		auto const format = GetReceivedFormat();
 		if (format == m_pFileDataObject->GetFormat()) {
 			m_pLocalListView->m_state.HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
 		}
-		else if (format == m_pLocalDataObject->GetFormat()) {
-			m_pLocalListView->m_state.HandleDroppedFiles(m_pLocalDataObject, dir, def == wxDragCopy);
+		else if (format == LocalDataObjectFormat()) {
+			m_pLocalListView->m_state.HandleDroppedFiles(GetLocalDataObject(), dir, def == wxDragCopy);
 		}
 		else {
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
+			auto * obj = GetRemoteDataObject();
+			if (obj->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.GetSite() || m_pRemoteDataObject->GetSite().server != m_pLocalListView->m_state.GetSite().server) {
+			if (!m_pLocalListView->m_state.GetSite() || obj->GetSite().server != m_pLocalListView->m_state.GetSite().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, dir)) {
+			if (!m_pLocalListView->m_state.DownloadDroppedFiles(obj, dir)) {
 				return wxDragNone;
 			}
 		}
@@ -129,7 +135,7 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
-		CScrollableDropTarget<wxListCtrlEx>::OnDrop(x, y);
+		CFileDropTarget::OnDrop(x, y);
 		ClearDropHighlight();
 
 		if (m_pLocalListView->m_fileData.empty()) {
@@ -139,7 +145,7 @@ public:
 		return true;
 	}
 
-	virtual int DisplayDropHighlight(wxPoint point)
+	virtual int DisplayDropHighlight(wxPoint const& point) override
 	{
 		DoDisplayDropHighlight(point);
 		return -1;
@@ -149,7 +155,7 @@ public:
 	{
 		wxString subDir;
 
-		int flags;
+		int flags{};
 		int hit = m_pLocalListView->HitTest(point, flags, 0);
 		if (!(flags & wxLIST_HITTEST_ONITEM)) {
 			hit = -1;
@@ -192,7 +198,7 @@ public:
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		def = CScrollableDropTarget<wxListCtrlEx>::OnDragOver(x, y, def);
+		def = CFileDropTarget::OnDragOver(x, y, def);
 
 		if (def == wxDragError ||
 			def == wxDragNone ||
@@ -231,13 +237,13 @@ public:
 
 	virtual void OnLeave()
 	{
-		CScrollableDropTarget<wxListCtrlEx>::OnLeave();
+		CFileDropTarget::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		def = CScrollableDropTarget<wxListCtrlEx>::OnEnter(x, y, def);
+		def = CFileDropTarget::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -256,6 +262,7 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_DELETE"), CLocalListView::OnMenuDelete)
 	EVT_MENU(XRCID("ID_RENAME"), CLocalListView::OnMenuRename)
 	EVT_KEY_DOWN(CLocalListView::OnKeyDown)
+	EVT_SIZE(CLocalListView::OnSize)
 	EVT_LIST_BEGIN_DRAG(wxID_ANY, CLocalListView::OnBeginDrag)
 	EVT_MENU(XRCID("ID_OPEN"), CLocalListView::OnMenuOpen)
 	EVT_MENU(XRCID("ID_EDIT"), CLocalListView::OnMenuEdit)
@@ -267,18 +274,18 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue)
-	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
+CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue, COptionsBase & options)
+	: CFileListCtrl<CLocalFileData>(pParent, pQueue, options),
 	CStateEventHandler(state),
 	m_parentView(pParent)
 {
-	wxGetApp().AddStartupProfileRecord("CLocalListView::CLocalListView");
+	wxGetApp().AddStartupProfileRecord("CLocalListView::CLocalListView"sv);
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	m_state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
-	const unsigned long widths[4] = { 120, 80, 100, 120 };
+	const unsigned long widths[4] = { 170, 80, 120, 120 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
@@ -297,12 +304,14 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 	EnablePrefixSearch(true);
 
 	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
+
+	m_pInfoText = new CInfoText(*this);
 }
 
 CLocalListView::~CLocalListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
+	options_.set(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
 	volumeEnumeratorThread_.reset();
@@ -383,11 +392,26 @@ regular_dir:
 		CStateFilterManager const& filter = m_state.GetStateFilterManager();
 		fz::local_filesys local_filesys;
 
-		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
+		auto result = local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false);
+		if (!result) {
+
+			if (result.error_ == fz::result::noperm) {
+				SetInfoText(_("You do not have permission to list this directory"));
+			}
+			else {
+				SetInfoText(_("Could not list directory contents"));
+			}
+
 			SetItemCount(1);
+			if (m_pFilelistStatusBar) {
+				m_pFilelistStatusBar->SetDirectoryContents(0, 0, 0, 0, 0);
+			}
+
 			return false;
 		}
 
+		SetInfoText(wxString());
+
 		int64_t totalSize{};
 		int unknown_sizes = 0;
 		int totalFileCount = 0;
@@ -396,10 +420,12 @@ regular_dir:
 
 		int num = m_fileData.size();
 		CLocalFileData data;
-		bool wasLink;
+		bool wasLink{};
+		fz::local_filesys::type t{};
 		fz::native_string name;
-		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+		while (local_filesys.get_next_file(name, wasLink, t, &data.size, &data.time, &data.attributes)) {
 			data.name = fz::to_wstring(name);
+			data.dir = t == fz::local_filesys::dir;
 			if (name.empty() || data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
@@ -408,23 +434,23 @@ regular_dir:
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir) {
-					totalDirCount++;
+					++totalDirCount;
 				}
 				else {
 					if (data.size != -1) {
 						totalSize += data.size;
 					}
 					else {
-						unknown_sizes++;
+						++unknown_sizes;
 					}
-					totalFileCount++;
+					++totalFileCount;
 				}
 				m_indexMapping.push_back(num);
 			}
 			else {
-				hidden++;
+				++hidden;
 			}
-			num++;
+			++num;
 		}
 
 		if (m_pFilelistStatusBar) {
@@ -525,7 +551,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	}
 
 	if (data->dir) {
-		const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+		const int action = options_.get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 		if (action == 3) {
 			// No action
 			wxBell();
@@ -557,7 +583,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_FILE);
+	const int action = options_.get_int(OPTION_DOUBLECLICK_ACTION_FILE);
 	if (action == 3) {
 		// No action
 		wxBell();
@@ -622,6 +648,7 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &)
 #ifdef __WXMSW__
 void CLocalListView::DisplayDrives()
 {
+	SetInfoText(wxString());
 	int count = m_fileData.size();
 
 	std::vector<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
@@ -657,6 +684,8 @@ void CLocalListView::DisplayDrives()
 
 void CLocalListView::DisplayShares(wxString computer)
 {
+	SetInfoText(wxString());
+
 	// Cast through a union to avoid warning about breaking strict aliasing rule
 	union
 	{
@@ -736,37 +765,37 @@ bool CLocalListView::IsItemValid(unsigned int item) const
 	return true;
 }
 
-std::unique_ptr<CFileListCtrlSortBase> CLocalListView::GetSortComparisonObject()
+void CLocalListView::UpdateSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
-	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
+	NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection) {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 	else {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 }
@@ -781,29 +810,29 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	wxMenu menu;
 
 	auto item = new wxMenuItem(&menu, XRCID("ID_UPLOAD"), _("&Upload"), _("Upload selected files and directories"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_UPLOAD"), wxART_MENU)));
 	menu.Append(item);
 	item = new wxMenuItem(&menu, XRCID("ID_ADDTOQUEUE"), _("&Add files to queue"), _("Add selected files and folders to the transfer queue"));
-	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
+	item->SetBitmap(MakeBmpBundle(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU)));
 	menu.Append(item);
 	menu.Append(XRCID("ID_ENTER"), _("E&nter directory"), _("Enter selected directory"));
-	
+
 	menu.AppendSeparator();
-	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open the file."));		
+	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open the file."));
 	menu.Append(XRCID("ID_EDIT"), _("&Edit"), _("Edit the file with the configured editor and upload changes to the server."));
-		
+
 	menu.AppendSeparator();
-	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));		
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
 	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
 	menu.Append(XRCID("ID_CONTEXT_REFRESH"), _("Re&fresh"));
-		
+
 	menu.AppendSeparator();
 	menu.Append(XRCID("ID_DELETE"),_("&Delete"), _("Delete selected files and directories"));
 	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected files and directories"));
-		
+
 	const bool connected = m_state.IsRemoteConnected();
 	if (!connected) {
-		menu.Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
+		menu.Enable(XRCID("ID_EDIT"), options_.get_int(OPTION_EDIT_TRACK_LOCAL) == 0);
 		menu.Enable(XRCID("ID_UPLOAD"), false);
 		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -926,7 +955,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	if (!root.empty()) {
 		recursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
+		recursiveOperation->StartRecursiveOperation(recursive_operation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 	}
 }
 
@@ -1157,7 +1186,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 void CLocalListView::ApplyCurrentFilter()
 {
 	CStateFilterManager const& filter = m_state.GetStateFilterManager();
-	
+
 	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
 	}
@@ -1361,6 +1390,14 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 	}
 }
 
+void CLocalListView::OnSize(wxSizeEvent& event)
+{
+	event.Skip();
+	if (m_pInfoText) {
+		m_pInfoText->Reposition();
+	}
+}
+
 void CLocalListView::OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR) {
@@ -1370,7 +1407,12 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, std
 		ApplyCurrentFilter();
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		if (m_windowTinter) {
+			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
+		if (m_pInfoText) {
+			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
@@ -1378,9 +1420,25 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, std
 	}
 }
 
+void CLocalListView::SetInfoText(wxString const& text)
+{
+	if (!m_pInfoText) {
+		return;
+	}
+
+	if (IsComparing() || text.empty()) {
+		m_pInfoText->Hide();
+	}
+	else {
+		m_pInfoText->SetText(text);
+		m_pInfoText->Reposition();
+		m_pInfoText->Show();
+	}
+}
+
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (options_.get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -1406,6 +1464,7 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
 	pDragDropManager->localParent = m_dir;
+	pDragDropManager->dragDataObject = &obj;
 
 	auto const path = m_dir.GetPath();
 
@@ -1439,12 +1498,11 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 
 	CLabelEditBlocker b(*this);
 
-	wxDropSource source(this);
+	DropSource source(this);
 	source.SetData(obj);
-	int res = source.DoDragDrop(wxDrag_AllowMove);
+	int res = source.DoFileDragDrop(wxDrag_AllowMove);
 
 	bool handled_internally = pDragDropManager->pDropTarget != 0;
-
 	pDragDropManager->Release();
 
 	if (!handled_internally && (res == wxDragCopy || res == wxDragMove)) {
@@ -1452,6 +1510,13 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 		// externally, the internal handlers do this for us already
 		m_state.RefreshLocal();
 	}
+
+#ifdef __WXMAC__
+	if (!source.m_OutDir.IsEmpty()) {
+		CLocalPath target(source.m_OutDir.ToStdWstring());
+		m_state.HandleDroppedFiles(&obj, target, true);
+	}
+#endif
 }
 
 void CLocalListView::RefreshFile(std::wstring const& file)
@@ -1573,7 +1638,7 @@ void CLocalListView::RefreshFile(std::wstring const& file)
 	if (m_hasParent) {
 		++start;
 	}
-	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
+	auto& compare = GetSortComparisonObject();
 	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, SortPredicate(compare));
 
 	const int item = insertPos - m_indexMapping.begin();
@@ -1659,7 +1724,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
+bool CLocalListView::get_next_file(std::wstring_view & name, std::wstring &, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -1682,6 +1747,8 @@ bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size
 
 void CLocalListView::FinishComparison()
 {
+	SetInfoText(wxString());
+
 	SetItemCount(m_indexMapping.size());
 
 	ComparisonRestoreSelections();
@@ -1689,11 +1756,9 @@ void CLocalListView::FinishComparison()
 	RefreshListOnly();
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther) {
-		return;
+	if (pOther) {
+		pOther->ScrollTopItem(GetTopItem());
 	}
-
-	pOther->ScrollTopItem(GetTopItem());
 }
 
 bool CLocalListView::CanStartComparison()
@@ -1750,7 +1815,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	CServerPath path;
 
 	if (!m_state.GetSite()) {
-		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
+		if (options_.get_int(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
@@ -1836,7 +1901,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 	}
 
 	if (selected_item_list.size() > 10) {
-		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno, options_);
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
 
@@ -1857,33 +1922,30 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 			continue;
 		}
 
+
 		wxFileName fn(m_dir.GetPath(), data.name);
 		if (wxLaunchDefaultApplication(fn.GetFullPath(), 0)) {
 			continue;
 		}
-		bool program_exists = false;
-		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
-		if (cmd.empty()) {
-			auto pos = data.name.find('.');
-			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
-				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
-			}
+		auto cmd_with_args = GetSystemAssociation(fn.GetFullPath().ToStdWstring());
+		if (cmd_with_args.empty()) {
+			cmd_with_args = pEditHandler->GetAssociation(fn.GetFullPath().ToStdWstring());
 		}
-		if (cmd.empty()) {
+		if (cmd_with_args.empty()) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 			continue;
 		}
-		if (!program_exists) {
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
+		if (!ProgramExists(cmd_with_args.front())) {
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd_with_args.front());
 			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
 			continue;
 		}
 
-		if (wxExecute(cmd)) {
+		if (fz::spawn_detached_process(AssociationToCommand(cmd_with_args, fn.GetFullPath().ToStdWstring()))) {
 			continue;
 		}
-
 		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
+
 	}
 }
 
