@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "sitemanager_dialog.h"
 
 #include "asksavepassworddialog.h"
@@ -7,7 +7,7 @@
 #include "drop_target_ex.h"
 #include "filezillaapp.h"
 #include "inputdialog.h"
-#include "ipcmutex.h"
+#include "loginmanager.h"
 #include "Options.h"
 #include "sitemanager_site.h"
 #include "themeprovider.h"
@@ -18,12 +18,19 @@
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
 
+#include "../commonui/fz_paths.h"
+#include "../commonui/ipcmutex.h"
+
+#include <wx/dirdlg.h>
 #include <wx/dnd.h>
-#include <wx/file.h>
+#include <wx/filedlg.h>
+#include <wx/menu.h>
 #include <wx/statline.h>
+#include <wx/wupdlock.h>
 
 #include <algorithm>
 #include <array>
+#include <list>
 
 BEGIN_EVENT_TABLE(CSiteManagerDialog, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CSiteManagerDialog::OnOK)
@@ -95,6 +102,11 @@ public:
 		m_pSiteManager = pSiteManager;
 	}
 
+	virtual wxTreeItemId DisplayDropHighlight(wxPoint const& p) override
+	{
+		return m_pSiteManager->tree_->DisplayDropHighlight(p);
+	}
+
 	bool IsValidDropLocation(wxTreeItemId const& hit, wxDragResult const& def)
 	{
 		if (!hit) {
@@ -135,7 +147,7 @@ public:
 			wxTreeItemId cur = hit;
 			while (cur && cur != tree->GetRootItem()) {
 				if (cur == item) {
-					ClearDropHighlight();
+					tree->ClearDropHighlight();
 					return wxDragNone;
 				}
 				cur = tree->GetItemParent(cur);
@@ -152,7 +164,8 @@ public:
 
 	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		ClearDropHighlight();
+		auto tree = m_pSiteManager->tree_;
+		tree->ClearDropHighlight();
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -160,7 +173,7 @@ public:
 			return def;
 		}
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, def)) {
 			return wxDragNone;
 		}
@@ -181,9 +194,11 @@ public:
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y);
-		ClearDropHighlight();
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		auto tree = m_pSiteManager->tree_;
+		tree->ClearDropHighlight();
+
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, wxDragCopy)) {
 			return wxDragNone;
 		}
@@ -194,7 +209,7 @@ public:
 	virtual void OnLeave()
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
-		ClearDropHighlight();
+		m_pSiteManager->tree_->ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
@@ -203,72 +218,36 @@ public:
 		return OnDragOver(x, y, def);
 	}
 
-	wxTreeItemId GetHit(const wxPoint& point)
-	{
-		int flags = 0;
-
-		wxTreeItemId hit = m_pSiteManager->tree_->HitTest(point, flags);
-
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
-			return wxTreeItemId();
-		}
-
-		return hit;
-	}
-
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
 		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
 
+		auto tree = m_pSiteManager->tree_;
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
 		{
-			ClearDropHighlight();
+			tree->ClearDropHighlight();
 			return def;
 		}
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, def)) {
-			ClearDropHighlight();
+			m_pSiteManager->tree_->ClearDropHighlight();
+		}
+		else {
+			tree->DisplayDropHighlight(hit);
 		}
-
-		DisplayDropHighlight(wxPoint(x, y));
 
 		return def;
 	}
 
-	void ClearDropHighlight()
-	{
-		if (m_dropHighlight == wxTreeItemId()) {
-			return;
-		}
-
-		auto* tree = m_pSiteManager->tree_;
-		tree->SetItemDropHighlight(m_dropHighlight, false);
-		m_dropHighlight = wxTreeItemId();
-	}
-
-	wxTreeItemId DisplayDropHighlight(wxPoint p)
-	{
-		ClearDropHighlight();
-
-		wxTreeItemId hit = GetHit(p);
-		if (hit.IsOk()) {
-			auto* tree = m_pSiteManager->tree_;
-			tree->SetItemDropHighlight(hit, true);
-			m_dropHighlight = hit;
-		}
-
-		return hit;
-	}
-
 protected:
 	CSiteManagerDialog* m_pSiteManager;
-	wxTreeItemId m_dropHighlight;
 };
 
-CSiteManagerDialog::CSiteManagerDialog()
+CSiteManagerDialog::CSiteManagerDialog(COptionsBase & options)
+	: options_(options)
 {
 }
 
@@ -290,7 +269,7 @@ wxPanel * CreateBookmarkPanel(wxWindow* parent, DialogLayout const& lay)
 	main->AddGrowableCol(0);
 
 	main->Add(new wxStaticText(panel, -1, _("&Local directory:")));
-	
+
 	auto row = lay.createFlex(0, 1);
 	main->Add(row, lay.grow);
 	row->AddGrowableCol(0);
@@ -341,7 +320,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 
 	left->Add(new wxStaticText(this, wxID_ANY, _("&Select entry:")));
 
-	tree_ = new wxTreeCtrlEx(this, XRCID("ID_SITETREE"), wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN | wxTR_EDIT_LABELS | wxTR_HAS_BUTTONS | wxTR_MULTIPLE | wxTR_LINES_AT_ROOT);
+	tree_ = new wxTreeCtrlEx(this, XRCID("ID_SITETREE"), wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxBORDER_SUNKEN | wxTR_EDIT_LABELS | wxTR_MULTIPLE);
 	tree_->SetFocus();
 	left->Add(tree_, lay.grow)->SetProportion(1);
 
@@ -374,17 +353,17 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s, true));
 
 	tree_->AssignImageList(pImageList);
 
 	auto right = new wxBoxSizer(wxVERTICAL);
 	sides->Add(right, 1, wxLEFT|wxGROW, lay.gap);
 
-	m_pNotebook_Site = new CSiteManagerSite(*this);
+	m_pNotebook_Site = new CSiteManagerSite(*this, options_);
 	if (!m_pNotebook_Site->Load(this)) {
 		return false;
 	}
@@ -524,7 +503,7 @@ void CSiteManagerDialog::OnOK(wxCommandEvent&)
 
 	UpdateItem();
 
-	if (!CAskSavePasswordDialog::Run(this)) {
+	if (!CAskSavePasswordDialog::Run(this, options_)) {
 		return;
 	}
 
@@ -560,7 +539,7 @@ void CSiteManagerDialog::OnConnect(wxCommandEvent&)
 
 	UpdateItem();
 
-	if (!CAskSavePasswordDialog::Run(this)) {
+	if (!CAskSavePasswordDialog::Run(this, options_)) {
 		return;
 	}
 
@@ -574,15 +553,13 @@ void CSiteManagerDialog::OnConnect(wxCommandEvent&)
 class CSiteManagerXmlHandler_Tree : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Tree(wxTreeCtrlEx* tree_, wxTreeItemId root, std::wstring const& lastSelection, bool predefined)
-		: m_tree_(tree_), m_item(root), m_predefined(predefined)
+	CSiteManagerXmlHandler_Tree(wxTreeCtrlEx* tree_, wxTreeItemId root, std::wstring const& lastSelection, bool predefined, int kiosk)
+		: m_tree_(tree_), m_item(root), m_predefined(predefined), m_kiosk(kiosk)
 	{
 		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection)) {
 			m_lastSelection.clear();
 		}
 		m_lastSelectionIt = m_lastSelection.cbegin();
-
-		m_kiosk = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -629,7 +606,7 @@ public:
 			data->credentials.encrypted_ = fz::public_key();
 		}
 
-		std::wstring const name = data->server.GetName();
+		std::wstring const name = data->GetName();
 
 		CSiteManagerItemData* pData = new CSiteManagerItemData(std::move(data));
 		wxTreeItemId newItem = m_tree_->AppendItem(m_item, name, 2, 2, pData);
@@ -753,7 +730,7 @@ bool CSiteManagerDialog::Load()
 		return true;
 	}
 
-	std::wstring lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = options_.get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '0') {
 		if (lastSelection == _T("0")) {
 			tree_->SafeSelectItem(treeId);
@@ -765,7 +742,7 @@ bool CSiteManagerDialog::Load()
 	else {
 		lastSelection.clear();
 	}
-	CSiteManagerXmlHandler_Tree handler(tree_, treeId, lastSelection, false);
+	CSiteManagerXmlHandler_Tree handler(tree_, treeId, lastSelection, false, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
 	bool res = CSiteManager::Load(element, handler);
 
@@ -814,8 +791,8 @@ bool CSiteManagerDialog::Save(pugi::xml_node element, wxTreeItemId treeId)
 
 		bool res = Save(element, m_ownSites);
 
-		if (!xml.Save(false)) {
-			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (!xml.Save()) {
+			if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 				return res;
 			}
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), xml.GetFileName(), xml.GetError());
@@ -865,13 +842,11 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 			bookmarkChild = tree_->GetNextChild(child, cookie);
 		}
 
-		CSiteManager::Save(node, *data->m_site);
+		CSiteManager::Save(node, *data->m_site, CLoginManager::Get(), options_);
 
 		if (data->connected_item != -1) {
-			if ((*m_connected_sites)[data->connected_item].site.server == data->m_site->server) {
-				(*m_connected_sites)[data->connected_item].new_path = GetSitePath(child);
-				(*m_connected_sites)[data->connected_item].site = *data->m_site;
-			}
+			(*m_connected_sites)[data->connected_item].site = *data->m_site;
+			(*m_connected_sites)[data->connected_item].site.SetSitePath(GetSitePath(child));
 		}
 	}
 
@@ -880,7 +855,7 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 
 void CSiteManagerDialog::OnNewFolder(wxCommandEvent&)
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -922,8 +897,8 @@ bool CSiteManagerDialog::Verify()
 	}
 
 	if (data->m_site) {
-		bool const predefined = IsPredefinedItem(item);
-		if (!m_pNotebook_Site->Verify(predefined)) {
+		Site newSite = *data->m_site;
+		if (!m_pNotebook_Site->UpdateSite(newSite, false)) {
 			return false;
 		}
 	}
@@ -1053,7 +1028,7 @@ void CSiteManagerDialog::OnRename(wxCommandEvent&)
 
 void CSiteManagerDialog::OnDelete(wxCommandEvent&)
 {
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -1079,7 +1054,7 @@ void CSiteManagerDialog::OnDelete(wxCommandEvent&)
 		return;
 	}
 
-	CConditionalDialog dlg(this, CConditionalDialog::sitemanager_confirmdelete, CConditionalDialog::yesno);
+	CConditionalDialog dlg(this, CConditionalDialog::sitemanager_confirmdelete, CConditionalDialog::yesno, options_);
 	dlg.SetTitle(_("Delete Site Manager entries"));
 
 	dlg.AddText(_("Do you really want to delete the selected entries?"));
@@ -1113,7 +1088,7 @@ void CSiteManagerDialog::OnDelete(wxCommandEvent&)
 
 void CSiteManagerDialog::OnSelChanging(wxTreeEvent& event)
 {
-	if (m_is_deleting) {
+	if (m_is_deleting || tree_->ShouldIgnoreChangeEvent()) {
 		return;
 	}
 
@@ -1142,7 +1117,7 @@ void CSiteManagerDialog::OnSelChanging(wxTreeEvent& event)
 
 void CSiteManagerDialog::OnSelChanged(wxTreeEvent& evt)
 {
-	if (m_is_deleting) {
+	if (m_is_deleting || tree_->ShouldIgnoreChangeEvent()) {
 		return;
 	}
 
@@ -1152,12 +1127,20 @@ void CSiteManagerDialog::OnSelChanged(wxTreeEvent& evt)
 		m_is_deleting = false;
 	}
 
+#if !defined(__WXMSW__) && wxCHECK_VERSION(3, 2, 1)
+	auto focused = tree_->GetFocusedItem();
+	if (focused) {
+		tree_->EnsureVisible(focused);
+	}
+#endif
+
+	wxWindowUpdateLocker l(this);
 	SetCtrlState();
 }
 
 void CSiteManagerDialog::OnNewSite(wxCommandEvent&)
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 	if (selections.empty()) {
 		return;
 	}
@@ -1197,8 +1180,7 @@ bool CSiteManagerDialog::UpdateItem()
 	}
 
 	if (data->m_site) {
-		UpdateServer(*data->m_site, tree_->GetItemText(item));
-		return true;
+		return UpdateServer(*data->m_site, tree_->GetItemText(item));
 	}
 	else {
 		wxASSERT(data->m_bookmark);
@@ -1224,11 +1206,16 @@ bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, Site const& site)
 	return true;
 }
 
-void CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
+bool CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
 {
-	m_pNotebook_Site->UpdateSite(site);
+	Site newSite = site;
+	newSite.SetName(name.ToStdWstring());
+	if (!m_pNotebook_Site->UpdateSite(newSite, true)) {
+		return false;
+	}
 
-	site.server.SetName(name.ToStdWstring());
+	site = newSite;
+	return true;
 }
 
 bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
@@ -1298,7 +1285,7 @@ void CSiteManagerDialog::OnItemActivated(wxTreeEvent&)
 
 void CSiteManagerDialog::SetCtrlState()
 {
-	auto const selections = tree_->GetSelections();
+	auto const selections = tree_->GetAllSelections();
 
 	wxTreeItemId item;
 	if (selections.size() == 1) {
@@ -1405,22 +1392,34 @@ void CSiteManagerDialog::OnSearch(wxCommandEvent&)
 
 	wxString search = dlg.GetValue().Lower();
 
-	m_is_deleting = true;
-	tree_->UnselectAll();
-
 	bool match{};
-	wxTreeItemId item = tree_->GetRootItem();
-	while (item) {
-		auto name = tree_->GetItemText(item).Lower();
-		if (name.find(search) != wxString::npos) {
-			tree_->SafeSelectItem(item, false);
-			match = true;
-		}
 
-		item = tree_->GetNextItemSimple(item, true);
+	{
+		wxWindowUpdateLocker l(this);
+
+		m_is_deleting = true;
+		tree_->UnselectAll();
+
+		wxTreeItemId item = tree_->GetRootItem();
+		while (item) {
+			bool select{};
+			if (item != tree_->GetRootItem() && item != m_ownSites && item != m_predefinedSites) {
+				auto name = tree_->GetItemText(item).Lower();
+				if (name.find(search) != wxString::npos) {
+					select = true;
+				}
+			}
+
+			if (select) {
+				tree_->SafeSelectItem(item, false);
+				match = true;
+			}
+
+			item = tree_->GetNextItemSimple(item, true);
+		}
+		SetCtrlState();
+		m_is_deleting = false;
 	}
-	SetCtrlState();
-	m_is_deleting = false;
 
 	if (match) {
 		tree_->SetFocus();
@@ -1451,7 +1450,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent&)
 		items.push_back(item);
 	}
 	else {
-		auto selections = tree_->GetSelections();
+		auto selections = tree_->GetAllSelections();
 
 		wxTreeItemId ancestor;
 		for (auto const& item : selections) {
@@ -1493,7 +1492,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent&)
 
 bool CSiteManagerDialog::LoadDefaultSites()
 {
-	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	CLocalPath const defaultsDir = GetDefaultsDir();
 	if (defaultsDir.empty()) {
 		return false;
 	}
@@ -1518,7 +1517,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_Expanded);
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_SelectedExpanded);
 
-	std::wstring lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = options_.get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '1') {
 		if (lastSelection == _T("1")) {
 			tree_->SafeSelectItem(m_predefinedSites);
@@ -1530,7 +1529,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	else {
 		lastSelection.clear();
 	}
-	CSiteManagerXmlHandler_Tree handler(tree_, m_predefinedSites, lastSelection, true);
+	CSiteManagerXmlHandler_Tree handler(tree_, m_predefinedSites, lastSelection, true, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
 	CSiteManager::Load(element, handler);
 
@@ -1562,7 +1561,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	}
 #endif
 
-	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+	if (options_.get_int(OPTION_DND_DISABLED) != 0) {
 		event.Veto();
 		return;
 	}
@@ -1580,7 +1579,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	std::vector<wxTreeItemId> items;
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 
 	bool predefined{};
 
@@ -1842,6 +1841,10 @@ void CSiteManagerDialog::AddNewSite(wxTreeItemId parent, Site const& site, bool
 	CSiteManagerItemData* pData = new CSiteManagerItemData;
 	pData->m_site = std::make_unique<Site>();
 	*pData->m_site = site;
+
+	// Erase updated server info
+	pData->m_site->server = site.GetOriginalServer();
+	pData->m_site->originalServer.reset();
 	if (connected) {
 		pData->connected_item = 0;
 	}
@@ -1879,11 +1882,11 @@ void CSiteManagerDialog::RememberLastSelected()
 	if (sel) {
 		path = GetSitePath(sel);
 	}
-	
-	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, path);
+
+	options_.set(OPTION_SITEMANAGER_LASTSELECTED, path);
 }
 
-void CSiteManagerDialog::OnContextMenu(wxTreeEvent& event)
+void CSiteManagerDialog::OnContextMenu(wxTreeEvent&)
 {
 	if (!Verify()) {
 		return;
@@ -1917,12 +1920,12 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 
 	auto servers = exportRoot.append_child("Servers");
 
-	auto selections = tree_->GetSelections();
+	auto selections = tree_->GetAllSelections();
 
 	wxTreeItemId ancestor;
 	for (auto const& item : selections) {
-		if (!item.IsOk() || item == tree_->GetRootItem()) {
-			return;
+		if (!item.IsOk()) {
+			continue;
 		}
 
 		// Only keep items that do not have an ancestor that is already being copied
@@ -1932,11 +1935,16 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 		}
 		if (!parent) {
 			ancestor = item;
-			SaveChild(servers, item);
+			if (item == m_ownSites || item == m_predefinedSites) {
+				Save(servers, item);
+			}
+			else {
+				SaveChild(servers, item);
+			}
 		}
 	}
 
-	if (!xml.Save(false)) {
+	if (!xml.Save()) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), xml.GetFileName(), xml.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
